﻿/*
    Copyright © 2011 Cradlepoint, Inc. <www.cradlepoint.com>.  All rights reserved. 
    This file contains confidential information of Cradlepoint, Inc. and your 
    use of this file is subject to the Cradlepoint Software License Agreement 
    distributed with this file. Unauthorized reproduction or distribution of  
    this file is subject to civil and criminal penalties. 
*/
using System;
using System.Net;
using System.Net.Sockets;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Threading;
using System.Xml;

namespace RouterControl
{
    public delegate void poplateTextBoxDelegate(string text);
    
    public enum Requests { Authenticate = 0, Dial, Hangup, Reboot, KillFirst, GetInfo, GetStatus, GetConfig, GetDiag, SetConfig, Filters, FilterInit }
    public enum ConnectionStatus { Idle = 0, Conncecting, Connected, Disconneting, Connect_Error, Unauthorized, Unrecoverable, Reestablishing }

    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public static System.Windows.Threading.Dispatcher dispatcher;
        public const int HEADER_LENGTH = 20;

        private enum ConnectState { Connect, Cancel, Disconnect }
        private TcpClient m_client;

        public MainWindow()
        {
            InitializeComponent();
            ConnectButton.Focus();
            dispatcher = this.Dispatcher;
        }
        
        public void UpdateOutput(String text)
        {
            if (Dispatcher.CurrentDispatcher != MainWindow.dispatcher)
            {
                dispatcher.Invoke(new poplateTextBoxDelegate(UpdateOutput), new object[] { text });
            }
            else
            {
                Output.Text += String.Format("\n{0}", text);
                Output.ScrollToEnd();
            }
        }

        private void ConnectContinue()
        {
            try
            {
                if (null == m_client || !m_client.Connected)
                {
                    throw new Exception("Failed to Connect");
                }

                NetworkStream stream = m_client.GetStream();

                ConnectButton.Content = Enum.GetName(typeof(ConnectState), ConnectState.Disconnect);
                UpdateOutput("Connected\n");
                //Give send button focus.

                //Don't auto authenticate.
                /*Athenticate authWindow = new Athenticate(stream);
                if (true == authWindow.ShowDialog())
                    UpdateOutput("Authentication succeeded");
                else
                    UpdateOutput("Authentication failed");
                */
                RequestSelect.IsEnabled = true;
                SendButton.IsEnabled = true;
                SendButton.Focus();

            }
            catch (Exception e)
            {
                ConnectButton.Content = Enum.GetName(typeof(ConnectState), ConnectState.Connect);
                UpdateOutput("Failed to connect. Error" + e.Message);
            }
        }

        private void OnConnect(IAsyncResult ar)
        {
            try
            {
                //Can get here from cancel. Make sure we have a Client.
                if (m_client.Client != null)
                    dispatcher.Invoke(System.Windows.Threading.DispatcherPriority.Normal, new Action(ConnectContinue));
            }
            catch (Exception e)
            {
                UpdateOutput("Program Error: " + e.Message);
            }
        }

        private XmlElement GetInterfaceNode(XmlDocument xmlDoc, String name)
        {
            XmlNodeList ports = xmlDoc.GetElementsByTagName("Interface");
            foreach ( XmlElement node in ports)
            {
                XmlAttribute port = node.GetAttributeNode("Name");
                if (name == port.Value)
                    return node;
            }

            return null;
        }

        private void Dial(Object threadContext )
        {
            NetworkStream stream = threadContext as NetworkStream;
            try
            {
                Response response = null;
                XmlDocument xmlDoc = null;

                //Get status to see if we can dial
                Request req = new Request(Command.Get, OpCode.Status);
                if (!req.Send(stream))
                    return;

                xmlDoc = null;
                if (req != null)
                {
                    response = new Response(stream);
                    xmlDoc = response.getDoc();
                }

                //Parse out infor for USB1
                XmlElement port = GetInterfaceNode(xmlDoc, "USB1");
                Int16 deviceAttached = 0;
                ConnectionStatus connectionStatus = 0;
                if (port != null)
                {
                    deviceAttached = Int16.Parse(port.GetElementsByTagName("DevState")[0].InnerText);
                    if (deviceAttached != 1)
                    {
                        UpdateOutput("\nThere is no device attatched to the router");
                        return;
                    }

                    connectionStatus = (ConnectionStatus)Enum.Parse(typeof(ConnectionStatus), port.GetElementsByTagName("ConnState")[0].InnerText);
                    switch (connectionStatus)
                    {
                        case ConnectionStatus.Idle:
                            //Ready to dial.
                            break;
                        case ConnectionStatus.Conncecting:
                        case ConnectionStatus.Reestablishing:
                            UpdateOutput("\nModem already connecting");
                            return;
                        case ConnectionStatus.Connected:
                            UpdateOutput("\nModem already connected");
                            return;

                        case ConnectionStatus.Disconneting:
                            UpdateOutput("Must wait for modem to be disconnected before dialing");
                            return;

                        default:
                            UpdateOutput("Modem in Error State.");
                            return;
                    }
                }
                //Send connect command
                req = new Request(Command.Start, OpCode.Connect);
                if (!req.Send(stream))
                    return;

                //Get Dial Response
                ResultCode res = new Response(stream).Result;
                if (res != ResultCode.OK)
                {
                    UpdateOutput(String.Format("\nDial Failed. Error: {0}", res));
                    return;
                }

                //Wait for connect
                do
                {
                    UpdateOutput("\nConnecting");

                    //See if we connected
                    req = new Request(Command.Get, OpCode.Status);
                    if (!req.Send(stream))
                        return;

                    xmlDoc = null;
                    if (req != null)
                    {
                        response = new Response(stream);
                        xmlDoc = response.getDoc();
                        if (null == xmlDoc)
                            throw (new Exception("\nNo Response from device"));
                    }

                    port = GetInterfaceNode(xmlDoc, "USB1");
                    connectionStatus = (ConnectionStatus)Enum.Parse(typeof(ConnectionStatus), port.GetElementsByTagName("ConnState")[0].InnerText);
                    deviceAttached = Int16.Parse(port.GetElementsByTagName("DevState")[0].InnerText);

                    if (deviceAttached != 1)
                    {
                        throw (new Exception("\nThere is no device attatched"));
                    }

                    //Sleep for .25 seconds before trying status
                    System.Threading.Thread.Sleep(500);
                } while (connectionStatus != ConnectionStatus.Connected);

                UpdateOutput("\nModem connected");
            }
            catch (Exception e)
            {
                UpdateOutput("\nFailed to connect. Error:" + e.Message);
            }
            return;
        }

        private void Hangup(Object threadContext)
        {
            NetworkStream stream = threadContext as NetworkStream;

            Response response = null;
            XmlDocument xmlDoc = null
                ;
            //See if we are connected
            Request req = new Request(Command.Get, OpCode.Status);
            if (!req.Send(stream))
                return;

            xmlDoc = null;
            if (req != null)
            {
                response = new Response(stream);
                xmlDoc = response.getDoc();
            }

            XmlElement port = GetInterfaceNode(xmlDoc, "USB1");
            ConnectionStatus connectionStatus = 0;
            Int16 deviceAttached = 0;
            if (port != null)
            {
                connectionStatus = (ConnectionStatus)Enum.Parse(typeof(ConnectionStatus), port.GetElementsByTagName("ConnState")[0].InnerText);
                deviceAttached = Int16.Parse(port.GetElementsByTagName("DevState")[0].InnerText);

                if (deviceAttached != 1)
                {
                    UpdateOutput("\nThere is no device attatched to the router");
                    return;
                }

                switch (connectionStatus)
                {
                    case ConnectionStatus.Idle:
                        UpdateOutput("\nModem already disconnected");
                        return;
                    case ConnectionStatus.Conncecting:
                    case ConnectionStatus.Connected:
                    case ConnectionStatus.Reestablishing:
                        //ready to disconnect.
                        break;

                    case ConnectionStatus.Disconneting:
                        UpdateOutput("\nModem already disconnecting");
                        return;

                    default:
                        UpdateOutput("Modem in Error State.");
                        return;
                }
            }
            req = new Request(Command.End, OpCode.Connect);
            if (!req.Send(stream))
                return;

            //Get Dial Response
            ResultCode res = new Response(stream).Result;
            if (res != ResultCode.OK)
            {
                UpdateOutput(String.Format("\nDial Failed. Error: {0}", res));
                return;
            }

            do
            {
                UpdateOutput("\nDisconnecting");

                //See if we disconnected
                req = new Request(Command.Get, OpCode.Status);
                if (!req.Send(stream))
                    return;

                xmlDoc = null;
                if (req != null)
                {
                    response = new Response(stream);
                    xmlDoc = response.getDoc();
                }

                port = GetInterfaceNode(xmlDoc, "USB1");
                connectionStatus = (ConnectionStatus)Enum.Parse(typeof(ConnectionStatus), port.GetElementsByTagName("ConnState")[0].InnerText);
                deviceAttached = Int16.Parse(port.GetElementsByTagName("DevState")[0].InnerText);
                if (deviceAttached != 1)
                {
                    UpdateOutput("\nThere is no device attatched to the router");
                    return;
                }

                //Sleep for .25 seconds before trying status
                System.Threading.Thread.Sleep(500);
            } while (connectionStatus != ConnectionStatus.Idle);

            UpdateOutput("\nModem disconnected");
            return;
        }

        //Private methods
        private void ConnectButton_Click(object sender, RoutedEventArgs e)
        {
            try
            {
                Button b = sender as Button;

                if (ConnectState.Connect == (ConnectState)Enum.Parse(typeof(ConnectState), b.Content.ToString()))
                {
                    b.Content = Enum.GetName(typeof(ConnectState), ConnectState.Cancel);

                    UpdateOutput("Connecting...");
                    m_client = new TcpClient();

                    String strIPAddr = String.Format("{0}.{1}.{2}.{3}", ipAddr_octet_1.Text, ipAddr_octet_2.Text, ipAddr_octet_3.Text, ipAddr_octet_4.Text);
                    IPAddress ipAdd = System.Net.IPAddress.Parse(strIPAddr);
                    m_client.BeginConnect(ipAdd, 8890, OnConnect, null);
                }
                else
                {
                    b.Content = Enum.GetName(typeof(ConnectState), ConnectState.Connect);

                    m_client.Close();
                    UpdateOutput("\nDisconnected");
                    RequestSelect.IsEnabled = false;
                    SendButton.IsEnabled = false;

                    return;
                }
            }
            catch (Exception ex)
            {
                Console.WriteLine("error");
                UpdateOutput(String.Format("\nError: {0}", ex.Message));
            }

        }

        private void ClearButton_Click(object sender, RoutedEventArgs e)
        {
            Output.Text = String.Empty;
        }

        private void ExitButton_Click(object sender, RoutedEventArgs e)
        {
            if (m_client != null)
                m_client.Close();
            
            Close();
        }

        private void SendButton_Click(object sender, RoutedEventArgs e)
        {
            if (null == m_client || null == m_client.Client || !m_client.Connected)
            {
                ConnectButton.Content = Enum.GetName(typeof(ConnectState), ConnectState.Connect);
                UpdateOutput("Not Connected");
                return;
            }

            String name = RequestSelect.SelectedItem.ToString();
            Requests request = (Requests)Enum.Parse(typeof(Requests), name);
            //if (Enum.TryParse(name, out request)) .Net 4.5
            {
                Request req = null;
                NetworkStream stream = m_client.GetStream();

                switch (request)
                {
                    case Requests.Authenticate:
                        Athenticate authWindow = new Athenticate(stream);
                        if (true == authWindow.ShowDialog())
                            UpdateOutput("Authentication succeeded");
                        else
                            UpdateOutput("Authentication failed");

                        break;

                    case Requests.Dial:
                        //ToDo: Should have a stop
                        ThreadPool.QueueUserWorkItem(Dial, stream);
                        break;

                    case Requests.Hangup:
                        //ToDo: Should have a stop
                        ThreadPool.QueueUserWorkItem(Hangup, stream);
                        break;

                    case Requests.KillFirst:
                        req = new Request(Command.Set, OpCode.KillFirst);
                        if (!req.Send(stream))
                            return;
                        break;

                    case Requests.Reboot:
                        req = new Request(Command.Set, OpCode.Reboot);
                        if (!req.Send(stream))
                            return;

                        ConnectButton.Content = Enum.GetName(typeof(ConnectState), ConnectState.Connect);
                        break;

                    case Requests.GetInfo:
                        req = new Request(Command.Get, OpCode.Info);
                        if (!req.Send(stream))
                            return;
                        break;

                    case Requests.GetStatus:
                        req = new Request(Command.Get, OpCode.Status);
                        if (!req.Send(stream))
                            return;
                        break;

                    case Requests.GetDiag:
                        req = new Request(Command.Get, OpCode.Diag);
                        if (!req.Send(stream))
                            return;
                        break;

                    case Requests.GetConfig:
                        req = new Request(Command.Get, OpCode.Config);
                        if (!req.Send(stream))
                            return;
                        break;

                    case Requests.SetConfig:
                        req = new Request(Command.Set, OpCode.Config);
                        if (!req.Send(stream))
                            return;
                        break;

                    case Requests.Filters:

                        UpdateOutput("\nGetting port blocking rules...");
                            
                        req = new Request(Command.Set, OpCode.Filters, stream);
                        if (!req.Send(stream))
                            return;
                      break;

                    case Requests.FilterInit:
                        UpdateOutput("\nRe-initializing port blocking rules...");
                        req = new Request(Command.Set, OpCode.InitFilters);
                        if (!req.Send(stream))
                            return;
                      break;
                }

                if (req != null)
                {
                    Response response = new Response(stream);
                }
            }
        }

        private void RequestSelect_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (SendButton != null)
                SendButton.Focus();
        }

        private void addr_PreviewTextInput(object sender, TextCompositionEventArgs e)
        {
            TextBox tb = sender as TextBox;

            switch (e.Text)
            {
                case ".":
                    if (ipAddr_octet_4.IsFocused)
                        ipAddr_octet_1.Focus();
                    else
                        tb.MoveFocus(new TraversalRequest(FocusNavigationDirection.Next));

                    e.Handled = true;
                    break;
                default:
                    int result;
                    if (!int.TryParse(e.Text, out result))
                        e.Handled = true;
                    break;
            }
        }

        private void addr_TextChanged(object sender, TextChangedEventArgs e)
        {
            TextBox tb = sender as TextBox;

            int result;
            if (!int.TryParse(tb.Text, out result))
                return;

            if (255 < int.Parse(tb.Text))
            {
                tb.Text = "255";
                tb.SelectAll();
                tb.Focus();
                return;
            }

            if (3 == tb.Text.Length)
                tb.MoveFocus(new TraversalRequest(FocusNavigationDirection.Next));
        }

        private void addr_PreviewKeyDown(object sender, KeyEventArgs e)
        {
            TextBox tb = sender as TextBox;

            switch (e.Key)
            {
                case Key.Back:
                    if (0 == tb.CaretIndex)
                    {
                        tb.MoveFocus(new TraversalRequest(FocusNavigationDirection.Previous));
                        TextBox tbNew = FocusManager.GetFocusedElement(this) as TextBox;

                        //Seting the caretIndex deletes that character.
                        tbNew.CaretIndex = tbNew.Text.Length;
                    }
                    break;

                case Key.Enter:
                    tb.MoveFocus(new TraversalRequest(FocusNavigationDirection.Next));
                    break;
            }
        }
    }
}
