﻿/*
    Copyright © 2011 Cradlepoint, Inc. <www.cradlepoint.com>.  All rights reserved. 
    This file contains confidential information of Cradlepoint, Inc. and your 
    use of this file is subject to the Cradlepoint Software License Agreement 
    distributed with this file. Unauthorized reproduction or distribution of  
    this file is subject to civil and criminal penalties. 
*/
using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Net.Sockets;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Xml;

namespace RouterControl
{
    /// <summary>
    /// Interaction logic for OutboundFilters.xaml
    /// </summary>
    public partial class OutboundFilters : Window, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        protected virtual void RaisePropertyChanged(String propertyName)
        {
            if ((PropertyChanged != null))
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
     
        public class FilterData: INotifyPropertyChanged
        {
            public event PropertyChangedEventHandler PropertyChanged;

            protected virtual void RaisePropertyChanged(String propertyName)
            {
                if ((PropertyChanged != null))
                {
                    PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
                }
            }
            
            public Int16 Index { get; set; }
            private Boolean changed;
            public Boolean Changed { get { return changed; } set { changed = value; RaisePropertyChanged("Changed"); } }

            private Int16 used;
            public Int16 Used { get { return used; } set { used = value; RaisePropertyChanged("Used"); } }
            private Boolean enabled;
            public Boolean Enabled { get { return enabled; } set { enabled = value; RaisePropertyChanged("Enabled"); } }
            private Int16 protocol;
            public Int16 Protocol { get { return protocol; } set { protocol = value; RaisePropertyChanged("Protocol"); } }
            private Int16 portStart;
            public Int16 PortStart { get { return portStart; } set { portStart = value; RaisePropertyChanged("PortStart"); } }
            private Int16 portEnd;
            public Int16 PortEnd { get { return portEnd; } set { portEnd = value; RaisePropertyChanged("PortEnd"); } }
        }

        private const UInt16 FILTER_FIELDS = 5;

        ObservableCollection<FilterData> m_FilterCollection =
            new ObservableCollection<FilterData>();

        NetworkStream m_Stream;
        String m_Results = String.Empty;

        public OutboundFilters(NetworkStream stream)
        {
            InitializeComponent();

            Owner = (MainWindow)Application.Current.MainWindow;
            m_Stream = stream;
            CloseButton.Focus();

            Loading.IsEnabled = true;
            BackgroundWorker worker = new BackgroundWorker();
            worker.DoWork += new DoWorkEventHandler(GetRules);
            worker.RunWorkerCompleted += new RunWorkerCompletedEventHandler(delegate { Loading.IsEnabled = false; });
            worker.RunWorkerAsync();


            //ThreadPool.QueueUserWorkItem(GetRules);
        }

        public ObservableCollection<FilterData> FilterCollection
        { get { return new ObservableCollection<FilterData>(m_FilterCollection.Where(q => q.Used == 1)); }
        }
        private Boolean globalEnabled;
        public Boolean GlobalEnabled { get { return globalEnabled; } set { globalEnabled = value; RaisePropertyChanged("GlobalEnabled"); } }

        private Int16 globalAction;
        public Int16 GlobalAction { get { return globalAction; } set { globalAction = value; RaisePropertyChanged("GlobalAction"); } }

        public String ResponseText { get { return m_Results; } }

        //just setting one flag on either global varable clicked/changed
        private Boolean globalChanged = false;

        private void GetRules(Object sender, DoWorkEventArgs e)
        {
            BackgroundWorker worker = sender as BackgroundWorker;
            Request req = new Request(Command.Get, OpCode.Config, new String[] { "filterCount", "filtersEnabled", "filtersAction" });

            if (!req.Send(m_Stream))
                return;

            XmlDocument xmlDoc = null;

            if (req != null)
            {
                Response response = new Response(m_Stream);
                xmlDoc = response.getDoc();

                if (null == xmlDoc)
                    return;
            }
            try
            {
                XmlNodeList nodes = xmlDoc.GetElementsByTagName("Var");

                Int16 count = Int16.Parse(nodes[0].InnerText);
                GlobalEnabled = Boolean.Parse(nodes[1].InnerText);
                GlobalAction = Int16.Parse(nodes[2].InnerText);


                String[] vars = new String[FILTER_FIELDS * count];

                for (Int16 i = 0; i < count; i++)
                {
                    vars[i * FILTER_FIELDS] = String.Format("filterRule[{0}].used", i);
                    vars[i * FILTER_FIELDS + 1] = String.Format("filterRule[{0}].enabled", i);
                    vars[i * FILTER_FIELDS + 2] = String.Format("filterRule[{0}].protocol", i);
                    vars[i * FILTER_FIELDS + 3] = String.Format("filterRule[{0}].dest_port_start", i);
                    vars[i * FILTER_FIELDS + 4] = String.Format("filterRule[{0}].dest_port_end", i);


                }

                req = new Request(Command.Get, OpCode.Config, vars);

                if (!req.Send(m_Stream))
                    return;

                xmlDoc = null;

                if (req != null)
                {
                    Response response = new Response(m_Stream);
                    xmlDoc = response.getDoc();
                }
                nodes = xmlDoc.GetElementsByTagName("Var");

                for (Int16 i = 0; i < count; i++)
                {
                    Int16 used = Int16.Parse(nodes[i * FILTER_FIELDS].InnerText);

                    if (1 == used)
                    {
                        m_FilterCollection.Add(new FilterData
                        {
                            Index = i,
                            Used = used,
                            Enabled = Boolean.Parse(nodes[i * FILTER_FIELDS + 1].InnerText),
                            Protocol = Int16.Parse(nodes[i * FILTER_FIELDS + 2].InnerText),
                            PortStart = Int16.Parse(nodes[i * FILTER_FIELDS + 3].InnerText),
                            PortEnd = Int16.Parse(nodes[i * FILTER_FIELDS + 4].InnerText)
                        });

                        RaisePropertyChanged("FilterCollection");
                    }
                }
            }
            catch (Exception ex)
            {
               
            }
        }

        private void CloseButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = false;
            Close();
        }

        private void SaveButton_Click(object sender, RoutedEventArgs e)
        {
            StringBuilder sb = new StringBuilder();

            //Save global changed
            if (true == globalChanged)
            {
                sb.AppendFormat("<var name=\"filtersEnabled\">{0}</var>\n", globalEnabled);
                sb.AppendFormat("<var name=\"filtersAction\">{0}</var>\n", globalAction);
            }

            //need to Go through the list and look for changed items and generate String array of config variables/values 
            //to send to device.
            //How many changes
            foreach (FilterData data in m_FilterCollection)
            {
                if (!data.Changed)
                    continue;
                sb.AppendFormat("<var name=\"filterRule[{0}].used\">{1}</var>\n", data.Index, data.Used);
                sb.AppendFormat("<var name=\"filterRule[{0}].enabled\">{1}</var>\n", data.Index, (data.Enabled));
                sb.AppendFormat("<var name=\"filterRule[{0}].protocol\">{1}</var>\n", data.Index, (data.Protocol));
                sb.AppendFormat("<var name=\"filterRule[{0}].dest_port_start\">{1}</var>\n", data.Index, (data.PortStart));
                sb.AppendFormat("<var name=\"filterRule[{0}].dest_port_end\">{1}</var>\n", data.Index, (data.PortEnd));

            }
            m_Results = sb.ToString();
            DialogResult = true;
        }

        private void Protocol_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {

        }

        private void EditButton_Click(object sender, RoutedEventArgs e)
        {
            AddButton.Content = "Save";
            FilterData data = (FilterData)(sender as FrameworkElement).DataContext;

            //FilterData data = FilterBox.SelectedItem as FilterData;
            
            EnabledCB.IsChecked = data.Enabled;

            switch (data.Protocol)
            {
                case 6:
                    Protocol.SelectedIndex = 0;
                    break;
                case 17:
                    Protocol.SelectedIndex = 1;
                    break;
            }

            StartPort.Text = String.Format("{0}",data.PortStart);
            EndPort.Text = String.Format("{0}", data.PortEnd);
        }

        private void DeleteButton_Click(object sender, RoutedEventArgs e)
        {
            FilterData data = (FilterData)(sender as FrameworkElement).DataContext;

            //FilterCollection.RemoveAt(FilterBox.SelectedIndex);
            ClearButton_Click(sender, e);
            data.Used = 0;
            data.Changed = true;
            data.Enabled = false;
            data.PortStart = 0;
            data.PortEnd = 0;

            RaisePropertyChanged("FilterCollection");
        }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            //Validate Ports
            Int16 portStart = -1;
            Int16 portEnd = -1;
            if (!Int16.TryParse(StartPort.Text, out portStart) || portStart < 0 ||
                !Int16.TryParse(StartPort.Text, out portEnd) || portEnd < 0)
            {
                MessageBox.Show("Invalid Entry");
                return;
            }


            if (AddButton.Content.ToString() == "Save")
            {
                FilterData data = m_FilterCollection[FilterBox.SelectedIndex];

                m_FilterCollection[FilterBox.SelectedIndex].Enabled = (Boolean)EnabledCB.IsChecked;
                m_FilterCollection[FilterBox.SelectedIndex].Protocol = Int16.Parse((String)(Protocol.SelectedItem as ComboBoxItem).Tag);
                m_FilterCollection[FilterBox.SelectedIndex].PortStart = Int16.Parse(StartPort.Text);
                m_FilterCollection[FilterBox.SelectedIndex].PortEnd = Int16.Parse(EndPort.Text);
                ClearButton_Click(sender, e);
                m_FilterCollection[FilterBox.SelectedIndex].Changed = true;
            }
            else
            {
                Int16 new_index = (Int16)(m_FilterCollection[0].Index - 1);

                if (new_index < 0)
                {
                    MessageBox.Show("Table is full");
                    return;
                }


                m_FilterCollection.Insert(0, new FilterData
                {
                    //get next index from list.
                    Index = new_index,
                    Enabled = (Boolean)EnabledCB.IsChecked,
                    Protocol = Int16.Parse((String)(Protocol.SelectedItem as ComboBoxItem).Tag),

                    PortStart = portStart,
                    PortEnd = portEnd,
                    Changed = true,
                    Used = 1,
                });

                ClearButton_Click(sender, e);
                RaisePropertyChanged("FilterCollection");

            }
        }

        private void ClearButton_Click(object sender, RoutedEventArgs e)
        {
            EnabledCB.IsChecked = false;
            Protocol.SelectedIndex = 0;
            StartPort.Text = String.Empty;
            EndPort.Text = String.Empty;
            AddButton.Content = "Add";

        }

        private void CheckBox_Click(object sender, RoutedEventArgs e)
        {
            FilterData data = (FilterData)(sender as FrameworkElement).DataContext;
            data.Changed = true; ;
        }

        private void CheckBox_Checked(object sender, RoutedEventArgs e)
        {
            globalChanged = true;
        }

        private void GlobalAction_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            globalChanged = true;
        }
    }

    public class BoolToVisibilityConverter : IValueConverter
    {
        public object Convert(object value, Type targetType,
          object parameter, System.Globalization.CultureInfo culture)
        {
            bool param = bool.Parse(parameter as string);
            bool val = (bool)value;

            return val == param ?Visibility.Hidden : Visibility.Visible;
        }

        public object ConvertBack(object value, Type targetType,
          object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
}
