﻿/*
    Copyright © 2011 Cradlepoint, Inc. <www.cradlepoint.com>.  All rights reserved. 
    This file contains confidential information of Cradlepoint, Inc. and your 
    use of this file is subject to the Cradlepoint Software License Agreement 
    distributed with this file. Unauthorized reproduction or distribution of  
    this file is subject to civil and criminal penalties. 
*/
using System;
using System.Net.Sockets;
using System.Text;
using System.Windows;
using System.Windows.Threading;
using System.Xml;


namespace RouterControl
{
    public enum Command { None = 0, Auth, Get, Set, Start, End }
    public enum OpCode { None = 0, Config, Status, Info, Connect, Reboot, KillFirst, Diag, InitFilters, Filters }

    class Request
    {
        private const int HEADER_LENGTH = 20;
        private const Int32 VERSION = 2;

        private Command m_command;
        private OpCode m_opCode;
        private String m_data;
        private Byte[] m_baSendData;

        public Request(Command command, OpCode opCode, params Object[] Args)
        {
            m_command = command;
            m_opCode = opCode;
            m_data = String.Empty;

            switch (command)
            {
                case Command.Auth:
                    m_data = String.Format("<Auth><Credentials>{0}</Credentials></Auth>", Args[0]);

                    break;
                case Command.Get:
                    switch (opCode)
                    {
                        case OpCode.Info:
                            m_data = "<Info><Router/><Interface name=\"USB1\" refresh=\"1\"/></Info>";
                            break;
                        case OpCode.Status:
                            m_data = "<Status><Router/><Interface name=\"USB1\"/></Status>";
                            break;
                        case OpCode.Config:

                            if (0 == Args.Length)
                            {
                                GetConfig configWin = new GetConfig();
                                if (true == configWin.ShowDialog())
                                    m_data = String.Format("<Config>{0}</Config>", configWin.ResponseText);
                                else
                                    ClearRequest();
                            }
                            else
                            {
                                StringBuilder sb = new StringBuilder("<Config>");
                                foreach (String str in Args)
                                {
                                    sb.AppendFormat("<var name=\"{0}\"/>", str);
                                }

                                sb.Append("</Config>");
                                m_data = sb.ToString();
                            }
                            break;
                        case OpCode.Diag:
                            GetDiagnostics diagWin = new GetDiagnostics();
                            if (true == diagWin.ShowDialog())
                                m_data = String.Format("<Diag><Interface name=\"USB1\" refresh=\"1\">{0}</Interface></Diag>", diagWin.ResponseText);
                            else
                                ClearRequest();

                            break;
                    }
                    break;
                case Command.Set:
                    switch (opCode)
                    {
                        case OpCode.Config:
                            SetConfig configWin = new SetConfig();
                            if (true == configWin.ShowDialog())
                                m_data = String.Format("<Config>{0}</Config>", configWin.ResponseText);
                            else
                                ClearRequest();

                            break;


                        case OpCode.Filters:
                            if (!(Args[0] is NetworkStream))
                                return;

                            //Set to config for setting values.
                            m_opCode = OpCode.Config;

                            OutboundFilters filterWin = new OutboundFilters(Args[0] as NetworkStream);
                            if (true == filterWin.ShowDialog())
                                    m_data = String.Format("<Config>{0}</Config>", filterWin.ResponseText);
                                else
                                    ClearRequest();
                            break;

                        case OpCode.InitFilters:

                            break;
                    }
                    break;
                case Command.Start:
                    if (OpCode.Connect == opCode)
                        m_data = "<Interface name =\"USB1\"/>";

                    break;
                case Command.End:
                    if (OpCode.Connect == opCode)
                        m_data = "<Interface name =\"USB1\"/>";
                    break;
            }


            //if (m_data != null)
                m_baSendData = build();
        }

        private void ClearRequest()
        {
            m_command = Command.None;
            m_opCode = OpCode.None;
        }

        private void UpdateOutput(String text)
        {
            if (Dispatcher.CurrentDispatcher != MainWindow.dispatcher)
                MainWindow.dispatcher.Invoke((Action)(delegate {((MainWindow)Application.Current.MainWindow).UpdateOutput(text);}));
            else
                ((MainWindow)Application.Current.MainWindow).UpdateOutput(text);
        }

        public override string ToString()
        {
            StringBuilder sb = new StringBuilder();
            try
            {

                sb.AppendFormat("Request:  {0:d} ({1})\n", m_command, m_command);
                sb.AppendFormat("Opcode:   {0:d} ({1})\n", m_opCode, m_opCode);
                sb.AppendFormat("Version:  {0:d} (version)\n", VERSION);
                sb.AppendFormat("Result:   {0:d} (result code)\n", 0);
                sb.AppendFormat("Data Len: {0:d}\n", m_data.Length);


                if (0 < m_data.Length)
                {
                    StringBuilder sbXml = new StringBuilder();

                    sbXml.Append("<?xml version='1.0'?>");
                    sbXml.Append(m_data);

                    XmlDocument doc = new XmlDocument();
                    doc.LoadXml(sbXml.ToString());

                    //sbXml.Clear();.Net 4.5 
                    sbXml = new StringBuilder();

                    XmlWriterSettings settings = new XmlWriterSettings();
                    settings.Indent = true;
                    settings.IndentChars = "  ";
                    settings.NewLineChars = "\r\n";
                    settings.NewLineHandling = NewLineHandling.Replace;
                    XmlWriter writer = XmlWriter.Create(sbXml, settings);
                    doc.Save(writer);
                    writer.Close();

                    sb.Append(sbXml.ToString());

                }
            }
            catch (Exception e)
            {
                UpdateOutput("Program Error: " + e.Message);
                UpdateOutput("Request: " + m_data);
            }
            return sb.ToString();
        }


        public Boolean Send(NetworkStream stream)
        {
            try
            {
                //If the command is not valid just return true.
                if (m_command == Command.None)
                    return false;


                UpdateOutput("\nSending:\n" + ToString());
                
                stream.Write(m_baSendData, 0, m_baSendData.Length);
            }
            catch (Exception e)
            {
                UpdateOutput("Send Failed....\nError: " + e.Message);

                return false;
            }

            return true;
        }

        
        private Byte[] build()
        {
            Byte[] buffer = new Byte[HEADER_LENGTH + m_data.Length];


            Byte[] request_data = Encoding.ASCII.GetBytes(m_data);

            Byte[] ba_request = BitConverter.GetBytes((Int32)m_command);
            Byte[] ba_opcode = BitConverter.GetBytes((Int32)m_opCode);
            Byte[] ba_version = BitConverter.GetBytes((Int32)2);
            Byte[] ba_reply = BitConverter.GetBytes((Int32)0);
            Byte[] ba_data_length = BitConverter.GetBytes(m_data.Length);

            if (BitConverter.IsLittleEndian)
            {
                Array.Reverse(ba_request);
                Array.Reverse(ba_opcode);
                Array.Reverse(ba_version);
                Array.Reverse(ba_data_length);
            }

            Int32 Offset = 0;
            System.Buffer.BlockCopy(ba_request, 0, buffer, Offset, ba_request.Length);
            Offset += ba_request.Length;
            System.Buffer.BlockCopy(ba_opcode, 0, buffer, Offset, ba_opcode.Length);
            Offset += ba_opcode.Length;
            System.Buffer.BlockCopy(ba_version, 0, buffer, Offset, ba_version.Length);
            Offset += ba_version.Length;
            System.Buffer.BlockCopy(ba_reply, 0, buffer, Offset, ba_reply.Length);
            Offset += ba_reply.Length;
            System.Buffer.BlockCopy(ba_data_length, 0, buffer, Offset, ba_data_length.Length);
            Offset += ba_data_length.Length;

            System.Buffer.BlockCopy(request_data, 0, buffer, Offset, request_data.Length);

            return buffer;
        }

    }
}
