﻿/*
    Copyright © 2011 Cradlepoint, Inc. <www.cradlepoint.com>.  All rights reserved. 
    This file contains confidential information of Cradlepoint, Inc. and your 
    use of this file is subject to the Cradlepoint Software License Agreement 
    distributed with this file. Unauthorized reproduction or distribution of  
    this file is subject to civil and criminal penalties. 
*/
using System;
using System.Net.Sockets;
using System.Text;
using System.Threading;
using System.Windows;
using System.Windows.Threading;
using System.Xml;



namespace RouterControl
{
    public enum ResultCode { OK = 0, Fail_NoAuth, Fail_NoFirst, Fail_NoWan, Fail_NoConnect, Fail_Mode, Fail_Invalid, Fail_Syntax, Fail_Memory, Fail_ReqXML, Fail_Num }

    class Response
    {
        private Command m_command;
        private OpCode m_opCode;
        private Int32 m_version;
        private ResultCode m_resultCode;
        private Int32 m_dataLength;
        private XmlDocument m_result;
        private NetworkStream m_stream;
        private Int32 m_dataLeft;


        private String m_receive;
        private byte[] buffer = new byte[1024];
        private ManualResetEvent completedEvent;

        
        public Response(NetworkStream stream)
        {
            m_stream = stream;
            completedEvent = new ManualResetEvent(false);
            IAsyncResult ar = m_stream.BeginRead(buffer, 0, buffer.Length, ReadData, null);


            if (!completedEvent.WaitOne(30 * 1000, false))
            {
            }

            UpdateOutput("\n\nReceived:\n" + getResultText());
        }


        //Setup result property
        public ResultCode Result { get { return m_resultCode; } }

        private void UpdateOutput(String text)
        {
            if (Dispatcher.CurrentDispatcher != MainWindow.dispatcher)
                MainWindow.dispatcher.Invoke((Action)(delegate { ((MainWindow)Application.Current.MainWindow).UpdateOutput(text); }));
            else
                ((MainWindow)Application.Current.MainWindow).UpdateOutput(text);
        }


        private void ReadData(IAsyncResult ar)
        {
            try
            {
                if (m_stream.CanRead)
                {
                    Console.WriteLine("Reading Data");
                    int n = m_stream.EndRead(ar);
                    Console.WriteLine("Reading Data. Bytes: " + n);
                    int buffer_offset = 0;

                    Console.WriteLine("Reading Data. m_dataLeft: " + m_dataLeft);

                    if (m_dataLeft != 0)
                    {
                        if (m_dataLeft < n)
                        {
                            m_receive += Encoding.ASCII.GetString(buffer, 0, m_dataLeft);
                            buffer_offset = m_dataLeft;
                            n -= m_dataLeft;
                        }
                        else
                        {
                            m_receive += Encoding.ASCII.GetString(buffer, 0, n);
                            m_dataLeft -= n;
                            goto CheckDoc;
                            //m_stream.BeginRead(buffer, 0, buffer.Length, ReadData, null);
                            //return;
                        }
                    }


                        Byte[] rec = new Byte[4];

                        Buffer.BlockCopy(buffer, buffer_offset, rec, 0, 4);
                        if (BitConverter.IsLittleEndian)
                            Array.Reverse(rec);
                        m_command = (Command)BitConverter.ToInt32(rec, 0);

                        Buffer.BlockCopy(buffer, buffer_offset += 4, rec, 0, 4);
                        if (BitConverter.IsLittleEndian)
                            Array.Reverse(rec);
                        m_opCode = (OpCode)BitConverter.ToInt32(rec, 0);

                        Buffer.BlockCopy(buffer, buffer_offset += 4, rec, 0, 4);
                        if (BitConverter.IsLittleEndian)
                            Array.Reverse(rec);
                        m_version = BitConverter.ToInt32(rec, 0);

                        Buffer.BlockCopy(buffer, buffer_offset += 4, rec, 0, 4);
                        if (BitConverter.IsLittleEndian)
                            Array.Reverse(rec);
                        m_resultCode = (ResultCode)BitConverter.ToInt32(rec, 0);

                        Buffer.BlockCopy(buffer, buffer_offset += 4, rec, 0, 4);
                        if (BitConverter.IsLittleEndian)
                            Array.Reverse(rec);
                        m_dataLength = BitConverter.ToInt32(rec, 0);

                        if (0 == m_dataLength)
                        {
                            completedEvent.Set();
                            return;
                        }

                        m_receive += Encoding.ASCII.GetString(buffer, buffer_offset += 4, n - 20);
                        m_dataLeft = m_dataLength - n + 20;

CheckDoc:
                    if (ReadComplete())
                        completedEvent.Set();
                    else
                    {
                        if (m_resultCode == ResultCode.OK || 0 < m_dataLeft)
                            m_stream.BeginRead(buffer, 0, buffer.Length, ReadData, null);
                        else
                            completedEvent.Set();
                    }
                }
                else
                    Console.WriteLine("Cant Reading Data");

            }
            catch (Exception e)
            {
                HandleExceptions(e);
            }
        }

        private Boolean ReadComplete()
        {
            m_result = new XmlDocument();
            try
            {
                m_result.LoadXml(m_receive);
            }
            catch
            {
                return false;
            }
            return true;
        }

        private void HandleExceptions(Exception e)
        {
            completedEvent.Set();
            UpdateOutput("\n\nFAILED to read response: Error: " + e.Message);
        }

        public XmlDocument getDoc()
        {
            return m_result;
        }

        public String getResultText()
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendFormat("Request:  {0:d} ({1})\n", m_command, m_command);
            sb.AppendFormat("Opcode:   {0:d} ({1})\n", m_opCode, m_opCode);
            sb.AppendFormat("Version:  {0:d} ({1})\n", m_version, m_version);
            sb.AppendFormat("Result:   {0:d} ({1})\n", m_resultCode, m_resultCode);
            sb.AppendFormat("Data Len: {0:d}\n", m_dataLength);

            if (m_result != null)
            {
                sb.AppendLine("\nXML Resp:");

                StringBuilder sbXml = new StringBuilder();
                XmlWriterSettings settings = new XmlWriterSettings();
                settings.Indent = true;
                settings.IndentChars = "  ";
                settings.NewLineChars = "\r\n";
                settings.NewLineHandling = NewLineHandling.Replace;
                XmlWriter writer = XmlWriter.Create(sbXml, settings);
                m_result.Save(writer);
                writer.Close();
              

                sb = sb.Append(sbXml);
            }

            return sb.ToString();
        }
    }
}

