﻿using System;
using System.Net.Sockets;
using System.Text;
using System.Windows;
using System.Windows.Threading;
using System.Xml;


namespace RouterControl
{
    public enum ResultCode { OK = 0, Fail_NoAuth, Fail_NoFirst, Fail_NoWan, Fail_NoConnect, Fail_Mode, Fail_Invalid, Fail_Syntax, Fail_Memory, Fail_Num }

    class Response
    {
        private Command m_command;
        private OpCode m_opCode;
        private Int32 m_version;
        private ResultCode m_resultCode;
        private Int32 m_dataLength;
        private XmlDocument m_result;
        
        public Response(NetworkStream stream)
        {

            try
            {
                String results = String.Empty;

                while (stream.CanRead)

                {
                    Byte[] rec = new Byte[4];
                    int recieved = 0;

                    do
                    {
                        System.Threading.Thread.Sleep(1000);
                    } while (!stream.DataAvailable);

                    recieved = stream.Read(rec, 0, 4);
                    if (BitConverter.IsLittleEndian)
                        Array.Reverse(rec);
                    m_command = (Command)BitConverter.ToInt32(rec, 0);

                    recieved = stream.Read(rec, 0, 4);
                    if (BitConverter.IsLittleEndian)
                        Array.Reverse(rec);
                    m_opCode = (OpCode)BitConverter.ToInt32(rec, 0);

                    recieved = stream.Read(rec, 0, 4);
                    if (BitConverter.IsLittleEndian)
                        Array.Reverse(rec);
                    m_version = BitConverter.ToInt32(rec, 0);

                    recieved = stream.Read(rec, 0, 4);
                    if (BitConverter.IsLittleEndian)
                        Array.Reverse(rec);
                    m_resultCode = (ResultCode)BitConverter.ToInt32(rec, 0);

                    recieved = stream.Read(rec, 0, 4);
                    if (BitConverter.IsLittleEndian)
                        Array.Reverse(rec);
                    m_dataLength = BitConverter.ToInt32(rec, 0);

                    if (0 < m_dataLength)
                    {
                        rec = new Byte[m_dataLength];
                        recieved = stream.Read(rec, 0, m_dataLength);
                        String new_results = Encoding.ASCII.GetString(rec);
                        results += new_results;
                    }
                }

                m_result = new XmlDocument();
                try
                {
                    m_result.LoadXml(results);
                }
                catch (Exception e)
                {
                    UpdateOutput("Program Error: " + e.Message);
                }

                UpdateOutput("\n\nReceived:\n" + getResultText());
            }
            catch (Exception ex)
            {
                UpdateOutput("\n\nFAILED to read response: Error: " + ex.Message);
            }
        }

        //Setup result property
        public ResultCode Result { get { return m_resultCode; } }

        private void UpdateOutput(String text)
        {
            if (Dispatcher.CurrentDispatcher != MainWindow.dispatcher)
                MainWindow.dispatcher.Invoke((Action)(delegate { ((MainWindow)Application.Current.MainWindow).UpdateOutput(text); }));
            else
                ((MainWindow)Application.Current.MainWindow).UpdateOutput(text);
        }


        public XmlDocument getDoc()
        {
            return m_result;
        }

        public String getResultText()
        {
            StringBuilder sb = new StringBuilder();

            sb.AppendFormat("Request:  {0:d} ({1})\n", m_command, m_command);
            sb.AppendFormat("Opcode:   {0:d} ({1})\n", m_opCode, m_opCode);
            sb.AppendFormat("Version:  {0:d} ({1})\n", m_version, m_version);
            sb.AppendFormat("Result:   {0:d} ({1})\n", m_resultCode, m_resultCode);
            sb.AppendFormat("Data Len: {0:d}\n", m_dataLength);

            if (m_result != null)
            {
                sb.AppendLine("\nXML Resp:");

                StringBuilder sbXml = new StringBuilder();
                XmlWriterSettings settings = new XmlWriterSettings();
                settings.Indent = true;
                settings.IndentChars = "  ";
                settings.NewLineChars = "\r\n";
                settings.NewLineHandling = NewLineHandling.Replace;
                XmlWriter writer = XmlWriter.Create(sbXml, settings);
                m_result.Save(writer);
                writer.Close();
              

                sb = sb.Append(sbXml);
            }

            return sb.ToString();
        }
    }
}

