﻿/*
    Copyright © 2011 Cradlepoint, Inc. <www.cradlepoint.com>.  All rights reserved. 
    This file contains confidential information of Cradlepoint, Inc. and your 
    use of this file is subject to the Cradlepoint Software License Agreement 
    distributed with this file. Unauthorized reproduction or distribution of  
    this file is subject to civil and criminal penalties. 
*/
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Threading;
using System.Linq;
using System.Collections;
using System.Collections.Generic;


namespace RouterControl
{
    public enum ConfigVariable { All, userLogin, pppUsername, pppPassword, connectMode, pppIdleTime, dialScript}

    public enum BoolValues { False = 0, True }
    public enum ReconnectValues:int { Always = 0, Demand, Mannual}


    /// <summary>
    /// Interaction logic for SetConfig.xaml
    /// </summary>
    public partial class SetConfig : Window
    {
        static public Boolean m_userChanged = false;

        public SetConfig()
        {
            InitializeComponent();
            Owner = (MainWindow)Application.Current.MainWindow;
            DataObject.AddPastingHandler(ConfigValue, DontAllow);
            m_userChanged = false;
        }

        public String ResponseText { get { return ConfigBox.Text; } }

        private void AddButton_Click(object sender, RoutedEventArgs e)
        {
            String name = ConfigSelect.SelectedItem.ToString();
            ConfigVariable variable = (ConfigVariable)Enum.Parse(typeof(ConfigVariable), name);
            {
                switch (variable)
                {
                    case ConfigVariable.userLogin:
                        String temp = String.Format("<var name=\"{0}\"/>{1}\n", variable, ConfigValue.SelectedItem);
                        ConfigBox.Text += String.Format("<var name=\"{0}\">{1}</var>\n", variable, ConfigValue.SelectedItem);
                        break;

                    case ConfigVariable.connectMode:
                        temp = String.Format("<var name=\"{0}\"/>{1:d}\n", variable, (ConfigValue.SelectedValue));
                        ConfigBox.Text += String.Format("<var name=\"{0}\">{1:d}</var>\n", variable, ConfigValue.SelectedValue);
                        break;
                    case ConfigVariable.pppUsername:
                    case ConfigVariable.pppPassword:
                        if (null != ConfigValue.SelectedValue)
                        {
                            MessageBox.Show(this, "Please enter a user name", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                            return;
                        }
                        
                        temp = String.Format("<var name=\"{0}\"/>{1}</var>\n", variable, (ConfigValue.Text));
                        ConfigBox.Text += String.Format("<var name=\"{0}\">{1}</var>\n", variable, ConfigValue.Text);

                        break;
                    case ConfigVariable.pppIdleTime:
                        Int16 timeout;
                        if (!Int16.TryParse(ConfigValue.Text, out timeout) || timeout < 1 || 200 < timeout)
                        {
                            MessageBox.Show(this, "Please enter a value between 1 and 200 minutes", "Error", MessageBoxButton.OK, MessageBoxImage.Error);
                            ConfigValue.Focus();
                            return;
                        }

                        temp = String.Format("<var name=\"{0}\"/>{1}</var>\n", variable, (ConfigValue.Text));
                        ConfigBox.Text += String.Format("<var name=\"{0}\">{1}</var>\n", variable, ConfigValue.Text);
                        break;
                    case ConfigVariable.dialScript:
                        DialScript win = new DialScript(this);
                        if (true == win.ShowDialog())
                        {
                            ConfigBox.Text += String.Format("<var name=\"{0}\">{1}</var>\n", variable, win.ResponseText);
                        }
                        break;
                }
            }
        }

        private void CancelButton_Click(object sender, RoutedEventArgs e)
        {
            ConfigBox.Text = String.Empty;
            DialogResult = false;
            Close();
        }

        private void DoneButton_Click(object sender, RoutedEventArgs e)
        {
            DialogResult = true;
        }

        private void ClearButton_Click(object sender, RoutedEventArgs e)
        {
            ConfigBox.Text = String.Empty;
        }

        private void ConfigSelect_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            //Handle InitializeComponents call
            if (ConfigValue == null)
                return;

            switch ((ConfigVariable)ConfigSelect.SelectedItem)
            {
                case ConfigVariable.pppUsername:
                case ConfigVariable.pppPassword:
                case ConfigVariable.pppIdleTime:
                    ConfigValue.IsEditable = true;
                    break;
                case ConfigVariable.userLogin:
                case ConfigVariable.connectMode:
                case ConfigVariable.dialScript:
                    ConfigValue.IsEditable = false;
                    break;
            }
        }

        private void ConfigValue_Filter(object sender, FilterEventArgs e)
        {
            ConfigVariable item = (ConfigVariable)e.Item;
            if (ConfigVariable.All != item)
            {
                e.Accepted = true;
            }
            else
            {
                e.Accepted = false;
            }
        }

        private void Editable_Click(object sender, RoutedEventArgs e)
        {
            ConfigBox.IsReadOnly = (sender as CheckBox).IsChecked == true ? false : true;
        }
        
//        private void ConfigValue_PreviewTextInput(object sender, System.Windows.Input.TextCompositionEventArgs e)
        private void ConfigValue_ValidateTextInput(object sender, String s)
        {
            ComboBox cb = sender as ComboBox;
            switch ((ConfigVariable)ConfigSelect.SelectedItem)
            {
                case ConfigVariable.pppUsername:
                case ConfigVariable.pppPassword:
                    if (64 <= cb.Text.Length)
                    {
                        MessageBox.Show("Maximum length is 64 characters");
                        //e.Handled = true;
                        return;
                    }

                    break;
                case ConfigVariable.pppIdleTime:
                    UInt16 val;
                    //Clear out prompt
                    if (5 < cb.Text.Length)
                        cb.Text = String.Empty;

                   //if (!UInt16.TryParse(e.Text, out val) || !UInt16.TryParse(cb.Text + e.Text, out val))
                    if (!UInt16.TryParse(s, out val) || !UInt16.TryParse(cb.Text + s, out val))
                    {
                        MessageBox.Show("Enter a value Between 0 and " + UInt16.MaxValue);

                        //e.Handled = true;
                        return;
                    }

                    break;
            }
        }

        private void ConfigValue_PreviewTextInput(object sender, System.Windows.Input.TextCompositionEventArgs e)
        {
            m_userChanged = true;

           // ConfigValue_ValidateTextInput(sender, e.Text);
        }


        //Don't allow paste into value combo box.
        private void DontAllow(object sender, DataObjectPastingEventArgs e)
        {
            //DataObject d = (DataObject)e.DataObject;
            //String s = (String)d.GetData(System.Windows.DataFormats.Text);

            //ConfigValue_ValidateTextInput(sender, s);
            // e.CancelCommand();
            m_userChanged = true;

        }

        private void ConfigValue_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (false == m_userChanged)
            {
                //m_userChanged = true;
                return;
            }

            ComboBox cb = sender as ComboBox;

            List<TextChange> tc = e.Changes.Where(change => change.AddedLength > 0).ToList();
            if (tc.Count == 0)
                return;

            if (1 < tc.Count)
                MessageBox.Show("Got more than 1 change");

            //Get change from first item.
            String newText = cb.Text.Substring(tc[0].Offset, tc[0].AddedLength);

            switch ((ConfigVariable)ConfigSelect.SelectedItem)
            {
                case ConfigVariable.pppUsername:
                case ConfigVariable.pppPassword:
                    if (64 <= cb.Text.Length)
                    {
                        MessageBox.Show("Maximum length is 64 characters");
                        e.Handled = true;
                    }

                    break;
                case ConfigVariable.pppIdleTime:
                    UInt16 val;

                    if (!UInt16.TryParse(cb.Text, out val))
                    {
                        MessageBox.Show("Enter a value Between 0 and " + UInt16.MaxValue);
                        e.Handled = true;
                    }

                    break;
            }

            //Undo change. A preview change shure would be nice so this wouldn't cause this event to fire again.
            if (true == e.Handled)
            {
                m_userChanged = false;
                cb.Text = cb.Text.Remove(tc[0].Offset, tc[0].AddedLength);
            }
        }
    }
  

    public class PromptConverter : IMultiValueConverter
    {
        #region IValueConverter Members
        const String PromptUserName = "Enter user name";
        const String PromptPassword = "Enter password";
        const String PromptIdleTime = "Enter idle time in minutes";
        const String PromptDialString = "Press Add button to enter a dial script";


        public object Convert(object[] values, Type targetType, object parameter, System.Globalization.CultureInfo culture)
        {
            ConfigVariable configVar = (ConfigVariable)values[0];

            String[] temp = new String[1];

            //Reset use change to stop prompts from being validated.
            SetConfig.m_userChanged = false;

            switch (configVar)
            {
                case ConfigVariable.userLogin:
                    return Enum.GetValues(typeof(BoolValues));
                case ConfigVariable.connectMode:
                    return Enum.GetValues(typeof(ReconnectValues));
                case ConfigVariable.pppUsername:
                    temp[0] = PromptUserName;
                    return temp;
                case ConfigVariable.pppPassword:
                    temp[0] = PromptPassword;
                    return temp;

                case ConfigVariable.pppIdleTime:
                    temp[0] = PromptIdleTime;
                    return temp;
                case ConfigVariable.dialScript:
                    temp[0] = PromptDialString;
                    return temp;
            }
            return null;
        }

        public object[] ConvertBack(object value, Type[] targetTypes, object parameter, System.Globalization.CultureInfo culture)
        {
            throw new NotImplementedException();
        }
        #endregion


    }

}
